searchData={"items":[{"type":"module","title":"Jackalope","doc":"`Jackalope` is an opinionated MQTT library that simplifies the use of\n[`Tortoise311 MQTT`](https://hex.pm/packages/tortoise311) with cloud IoT\nservices.\n\nJackalope aims to make an interface that:\n\n- Ensure that important messages are delivered to the broker, by\n  having a local \"post office\" and tracking the in flight messages,\n  and implementing a concept of ttl (time to live) on the messages\n  placed in the mailbox; ensuring the \"request to unlock the door\"\n  won't happen two hours later when the MQTT connection finally\n  reconnects. This allows Jackalope to accept publish requests while\n  the connection is down.\n\nBesides this Jackalope aims to provide helpers for local testing,\nallowing you to test your application without having a connection to\nAWS; Jackalope should take care of that.","ref":"Jackalope.html"},{"type":"module","title":"Usage - Jackalope","doc":"The `Jackalope` module implements a `start_link/1` function; use this\nto start `Jackalope` as part of your application supervision tree. If\nproperly supervised it will allow you to start and stop `Jackalope`\nwith the part the application that needs MQTT connectivity.\n`Jackalope` is configured using a keyword list, consult the\n`Jackalope.start_link/1` documentation for information on the\navailable option values.\n\nOnce `Jackalope` is running it is possible to publish messages to the broker;\nin addition to this there are some connection specific functionality is exposed,\nallowing us to ask for the connection status, and request a connection reconnect.\n\n- `Jackalope.publish(topic, payload)` will publish a message to the\n  MQTT broker;\n\n- `Jackalope.reconnect()` will disconnect from the broker and\n  reconnect; this is useful if the device changes network connection.\n\nSubscriptions can be set as part of the connection options provided to Jackalope\nor added later on.\n\n- `Jackalope.subscribe(topic)` subscribes to a topic\n\n- `Jackalope.unsubscribe(topic)` unsubscribes from a topic\n\nPlease see the documentation for each of the functions for more\ninformation on usage; publish functions accept options such as setting quality of\nservice and time to live.","ref":"Jackalope.html#module-usage"},{"type":"function","title":"Jackalope.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Jackalope.html#child_spec/1"},{"type":"function","title":"Jackalope.connection_status/0","doc":"Get the current MQTT connection status","ref":"Jackalope.html#connection_status/0"},{"type":"function","title":"Jackalope.publish/3","doc":"Publish a message to the MQTT broker\n\nThe `payload` will get published on `topic`. `Jackalope` will keep\nthe message in a queue until we got a connection, at which point it\nwill dispatch the publish. This of course present us with a problem:\nwhat if we place a publish request to \"unlock the front door\" while\nthe client is offline? We don't want to receive a message that the\nfront door has been unlocked two hours later when the MQTT client\nreconnect; To solve that problem we have a `ttl` option we can\nspecify on the publish.\n\n```elixir\nJackalope.publish(\"doors/front_door\", %{action: \"unlock\"}, qos: 1, ttl: 5_000)\n```\n\nThe available publish options are:\n\n  - `qos` (default `1`) sets the quality of service of the message\n    delivery; Notice that only quality of service 0 an 1 are\n    supported by AWS IoT.\n\n  - `retain` (default `false`) sets whether the broker should retain the message.\n    Note that AWS IoT does not support this feature.\n\n  - `ttl` (default `3_600_000`) sets how long (in msecs) publishing the message will be\n    retried until it has expired.","ref":"Jackalope.html#publish/3"},{"type":"function","title":"Jackalope.reconnect/0","doc":"Request the MQTT client to reconnect to the broker\n\nThis can be useful on devices that has multiple network\ninterfaces.","ref":"Jackalope.html#reconnect/0"},{"type":"function","title":"Jackalope.start_link/1","doc":"Start a Jackalope session\n\nThis will start a supervised group of processes; part of the group\nwill keep track of the topic filter subscription state, and hold a\nlist of yet to be published messages, as well as the requested\nsubscription changes; the other part of the process tree will keep\nthe MQTT connection specific parts, making sure we got a\nconnection. See the main documentation on the `Jackalope` module for\nmore information on the process architecture.\n\n`Jackalope.start_link/1` takes a keyword list containing option\nvalues, that configure the instance, as an argument. The available\noptions and their defaults are:\n\n- `client_id` (default: \"jackalope\"), string that will be used as\n  the client_id of the MQTT connection; see `t Tortoise311.client_id`\n  for more information on valid client ids. Notice that the\n  client_id needs to be unique on the server, so two clients may not\n  have the same client_id.\n\n- `user_name` (optional) specifies the MQTT Username.\n\n- `password` (optional) specifies the MQTT Password. Google\n  cloud IOT requires a JWT for this field.\n\n- `initial_topics` (optional) specifies a list of topic_filters\n  Jackalope should connect to when a connection has been\n  established. Note that if the `Jackalope` supervisor is restarted\n  for any reason, any changes to topic subscriptions made via\n  `subscribe/1` or `unsubscribe/1` will be lost.\n\n- `clean_session` (default: true), a flag bit that controls the\n  life cycle of the session state. The MQTT session lasts only\n  as long as the network connection between client and server.\n  When the session ends, all session state in the server is deleted.\n\n- `handler` (default: `Jackalope.Handler.Logger`) specifies the\n  module implementing the callbacks (implementing\n  `Jackalope.Handler` behaviour) to use. This module reacts to\n  the events Jackalope communicates about the connection\n  life-cycle, including receiving a message on a\n  subscribed topic filter. Read the documentation for\n  `Jackalope.Handler` for more information on the events and\n  callbacks.\n\n- `server` (default: {Tortoise311.Transport.Tcp, [host: \"localhost\", port: 1883]}) specifies the\n  connection type, and its options, to use when connecting to the\n  MQTT server. The default specification will attempt to connect to\n  a broker running on localhost:1883, on an insecure\n  connection. This value should only be used for testing and\n  development.\n\n  Server options for use with AWS IoT:\n\n  [\n    verify: :verify_peer,\n    host: mqtt_host(), # must return the full name, *without wild cards*, for e.g. \"abcdefghijklmo-ats.iot.us-east-1.amazonaws.com\"\n    port: mqtt_port(), # must return the correct port, e.g. 443\n    alpn_advertised_protocols: [\"x-amzn-mqtt-ca\"],\n    server_name_indication: to_charlist(mqtt_host()),\n    cert: cert, # the device's X509 certificate in DER format\n    key: key, # the device's private key in DER format\n    cacerts: [signer_cert] ++ aws_ca_certs(), # the device's signer cert, plus AWS IoT CA certs in DER format to be returned by aws_ca_certs()\n    versions: [:\"tlsv1.2\"],\n    customize_hostname_check: [match_fun: :public_key.pkix_verify_hostname_match_fun(:https)]\n  ]\n\n- `work_list_mod` names the module implementing the Jackalope WorkList protocol that will be used to manage\n   the publish commands sent to Tortoise by the Jackalope Session.\n   The module must also implement the function `@spec new(function(), function(), non_neg_integer(), Keyword.t()) :: any()`.\n   See Jackalope.TransientWorkList (the default) for examples.\n\n- `max_work_list_size` (default: 100) specifies the maximum\n  number of unexpired work orders Jackalope will retain in its work list\n  (the commands yet to be sent to the MQTT server). When the maximum is\n  reached, the oldest work order is dropped before adding a new work order.\n\n- `last_will` (default: nil) specifies the last will message that\n  should get published on the MQTT broker if the connection is\n  closed or dropped unexpectedly. If we want to specify a last will\n  topic we should define a keyword list containing the following:\n\n    - `topic` (Required) the topic to post the last will message to;\n      this should be specified as a string and it should be a valid\n      MQTT topic; consult `t Tortoise311.topic` for more info on valid\n      MQTT topics.\n\n    - `payload` (default: nil) the payload of the last will message;\n\n    - `qos` (default: 0) either 0 or 1, denoting the quality of\n      service the last will message should get published with; note\n      that QoS=2 is not supported by AWS IoT.\n\n- `first_connect_delay` (optional) and passed to Tortoise311 if provided, else Tortoise311 uses its own default.\n   It specifies a delay in msecs before Tortoise311 connects for the first time to the MQTT server.\n   This can help spread out connections after an outage.\n\n- `backoff` (optional) and passed to Tortoise311 if provided, else Tortoise311 uses its own default.\n   It gives the bounds of an exponential backoff algorithm used\n   when Tortoise311 retries to reconnect from failed connections, or when it retries\n   to subscribe should connecting succeed but subsciptions fail (it can happen when an MQTT Server\n   throttles subscriptions but not connections).","ref":"Jackalope.html#start_link/1"},{"type":"function","title":"Jackalope.subscribe/1","doc":"Subscribe to a topic or list of topics.","ref":"Jackalope.html#subscribe/1"},{"type":"function","title":"Jackalope.subscriptions/0","doc":"Returns a list of topics the client is currently subscribed to.","ref":"Jackalope.html#subscriptions/0"},{"type":"function","title":"Jackalope.unsubscribe/1","doc":"Unsubscribe from a topic or list of topics.","ref":"Jackalope.html#unsubscribe/1"},{"type":"behaviour","title":"Jackalope.Handler","doc":"Behaviour defining callbacks triggered during the MQTT life-cycle\n\nThe jackalope handler is stateless, so if state is needed one could\nroute the messages to stateful processes, and inform the system\nabout connection and subscription state.\n\nMost of the callbacks are optional.","ref":"Jackalope.Handler.html"},{"type":"callback","title":"Jackalope.Handler.connection/1","doc":"Called when the MQTT connection changes status\n\nThis can be used to inform other parts of the system about the state\nof the connection; possible values are `:up` and `:down`, where up\nmeans that the MQTT client has a connection to the broker; down\nmeans that the connection has been dropped.","ref":"Jackalope.Handler.html#c:connection/1"},{"type":"callback","title":"Jackalope.Handler.handle_error/1","doc":"Handle errors produced by Jackalope that should be reacted to\n\nDuring the connection life-cycle various errors can occur, and while\nJackrabbit and Tortoise311 will try to correct the situation, some\nerrors require user intervention. The optional `handle_error/1`\ncallback can help inform the surrounding system of errors.\n\n    @impl Jackalope.Handler\n    def handle_error({:publish_error, work_order, :ttl_expired}) do\n      Logger.error(\"Work order expired: #{inspect(work_order)}\")\n    end\n\n    def handle_error(_otherwise) do\n      _ignore = nil\n    end\n\nIf this callback is implemented one should make sure to make a\ncatch-all to prevent unhandled errors from crashing the handler.","ref":"Jackalope.Handler.html#c:handle_error/1"},{"type":"callback","title":"Jackalope.Handler.handle_message/2","doc":"Called when receiving a message matching one of the subscriptions\n\nThe callback will receive two arguments; the MQTT topic in list\nform, where each of the topic levels are an item. This allows us to\npattern match on topic filters with wildcards.","ref":"Jackalope.Handler.html#c:handle_message/2"},{"type":"callback","title":"Jackalope.Handler.last_will/0","doc":"Produces the last will message for the current connection, or nil if the last will in the connection options is to be used\nExample: [topic: hub_serial_number/message\", payload: %{code: \"going_down\", msg: \"Last will message\"}, qos: 1]","ref":"Jackalope.Handler.html#c:last_will/0"},{"type":"callback","title":"Jackalope.Handler.subscription/2","doc":"Called when a topic filter subscription state changes\n\nThis can be used to inform other parts of the system that we should\n(or shouldn't) expect messages received on the given `topic_filter`.\n\nThe status values are `:up` and `:down`, where up means that the\nbroker has accepted a subscription request to the specific\n`topic_filter`, and down means that the broker has accepted an\nunsubscribe request.","ref":"Jackalope.Handler.html#c:subscription/2"},{"type":"type","title":"Jackalope.Handler.last_will/0","doc":"","ref":"Jackalope.Handler.html#t:last_will/0"},{"type":"type","title":"Jackalope.Handler.payload/0","doc":"","ref":"Jackalope.Handler.html#t:payload/0"},{"type":"type","title":"Jackalope.Handler.topic/0","doc":"","ref":"Jackalope.Handler.html#t:topic/0"},{"type":"type","title":"Jackalope.Handler.topic_filter/0","doc":"","ref":"Jackalope.Handler.html#t:topic_filter/0"},{"type":"type","title":"Jackalope.Handler.topic_levels/0","doc":"","ref":"Jackalope.Handler.html#t:topic_levels/0"},{"type":"module","title":"Jackalope.Handler.Logger","doc":"A `Jackalope.Handler` that logs everything it sees\n\nThis handler will be used by `Jackalope` if no `handler` has been\nspecified in the option list passed to `Jackalope.start_link/1`.","ref":"Jackalope.Handler.Logger.html"},{"type":"module","title":"Jackalope.TransientWorkList","doc":"A simple work list that does not survive across reboots.","ref":"Jackalope.TransientWorkList.html"},{"type":"function","title":"Jackalope.TransientWorkList.new/4","doc":"Create a new work list","ref":"Jackalope.TransientWorkList.html#new/4"},{"type":"type","title":"Jackalope.TransientWorkList.t/0","doc":"","ref":"Jackalope.TransientWorkList.html#t:t/0"},{"type":"protocol","title":"Jackalope.WorkList","doc":"A work list manages work items that have yet to be sent out for processing,\nor have been sent out and are waiting for confirmation of completion.","ref":"Jackalope.WorkList.html"},{"type":"function","title":"Jackalope.WorkList.count/1","doc":"Count the number of work items","ref":"Jackalope.WorkList.html#count/1"},{"type":"function","title":"Jackalope.WorkList.count_pending/1","doc":"Count the number of pending work items","ref":"Jackalope.WorkList.html#count_pending/1"},{"type":"function","title":"Jackalope.WorkList.done/2","doc":"Remove an item from the pending set","ref":"Jackalope.WorkList.html#done/2"},{"type":"function","title":"Jackalope.WorkList.empty?/1","doc":"Are there work items","ref":"Jackalope.WorkList.html#empty?/1"},{"type":"function","title":"Jackalope.WorkList.peek/1","doc":"Get the current item on the work list, if any. Leave it on the work list.","ref":"Jackalope.WorkList.html#peek/1"},{"type":"function","title":"Jackalope.WorkList.pending/2","doc":"Move a work item to the pending set as it is being worked on, awaiting confirmation of completion.","ref":"Jackalope.WorkList.html#pending/2"},{"type":"function","title":"Jackalope.WorkList.pop/1","doc":"Remove the current item from the work list","ref":"Jackalope.WorkList.html#pop/1"},{"type":"function","title":"Jackalope.WorkList.push/2","doc":"Push an item on the work list","ref":"Jackalope.WorkList.html#push/2"},{"type":"function","title":"Jackalope.WorkList.remove_all/1","doc":"Remove all work items","ref":"Jackalope.WorkList.html#remove_all/1"},{"type":"function","title":"Jackalope.WorkList.reset_pending/1","doc":"Empty out the set of pending items","ref":"Jackalope.WorkList.html#reset_pending/1"},{"type":"type","title":"Jackalope.WorkList.item/0","doc":"","ref":"Jackalope.WorkList.html#t:item/0"},{"type":"type","title":"Jackalope.WorkList.t/0","doc":"All the types that implement this protocol.","ref":"Jackalope.WorkList.html#t:t/0"},{"type":"type","title":"Jackalope.WorkList.work_list/0","doc":"","ref":"Jackalope.WorkList.html#t:work_list/0"},{"type":"module","title":"Jackalope.WorkList.Expiration","doc":"Common functions for work lists.","ref":"Jackalope.WorkList.Expiration.html"},{"type":"function","title":"Jackalope.WorkList.Expiration.after?/2","doc":"Does the first expiration come after the second?","ref":"Jackalope.WorkList.Expiration.html#after?/2"},{"type":"function","title":"Jackalope.WorkList.Expiration.expiration/1","doc":"","ref":"Jackalope.WorkList.Expiration.html#expiration/1"},{"type":"function","title":"Jackalope.WorkList.Expiration.now/0","doc":"","ref":"Jackalope.WorkList.Expiration.html#now/0"},{"type":"function","title":"Jackalope.WorkList.Expiration.rebase_expiration/3","doc":"Recalculate an old expiration given the latest time before stop and the earliest time after restart.\n  Assumes restart happened soon after stopping.","ref":"Jackalope.WorkList.Expiration.html#rebase_expiration/3"},{"type":"function","title":"Jackalope.WorkList.Expiration.unexpired?/2","doc":"","ref":"Jackalope.WorkList.Expiration.html#unexpired?/2"},{"type":"type","title":"Jackalope.WorkList.Expiration.expiration/0","doc":"","ref":"Jackalope.WorkList.Expiration.html#t:expiration/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"* Updates\n  * Make :first_connect_delay and :backoff optional Jackalope options (#104)\n\n* Fixes\n  * Pop expired items from the work list to avoid an infinite loop (#105)\n  * Fix session retry error analysis to account for timeout errors (#106)\n  * Bump dependencies and CI versions (#107)","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"v0.8.1 - Changelog","doc":"* Updates\n  * Upgraded to Tortoise311 v0.12.0","ref":"changelog.html#v0-8-1"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"* Updates\n  * Support for subscribing after connecting to the MQTT server","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"v0.7.3 - Changelog","doc":"* Updates\n  * Remove Logger warnings for Elixir 1.15\n\n* Fixes\n  * Add `:clean_session` option support. This fixes issues with QoS 1 sessions","ref":"changelog.html#v0-7-3"},{"type":"extras","title":"v0.7.2 - Changelog","doc":"* Updates\n  * Support querying the session's connection status","ref":"changelog.html#v0-7-2"},{"type":"extras","title":"v0.7.1 - Changelog","doc":"* Fixes\n  * Added dialyzer flags :missing_return, :extra_return\n  * Minor fixes to remove new dialyzer warnings\n  * Updated CI to OTP 25","ref":"changelog.html#v0-7-1"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"* Updated\n  * BREAKING CHANGE: Jackalope no longer encodes or decodes MQTT message payloads.","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"* Updates\n  * BREAKING CHANGE: Dynamic subscriptions are no longer supported\n  * :infinity is no longer the default for publish work item TTL or work list bound\n  * BREAKING CHANGE: The API for publishing is simplified; there's now only one publish call\n\n* Fixes\n  * Dynamic subscriptions were broken in failure and retry scenarios, removing support for them fixes that","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"v0.5.1 - Changelog","doc":"* Updates\n  * Uses backoff options","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"* Updates\n  * Bumped tortoise311 to 0.11","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"v0.4.2 - DEPRECATED - Changelog","doc":"* Updates\n  * All references to Tortoise are replaced by Tortoise311","ref":"changelog.html#v0-4-2-deprecated"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"* Updates\n  * Now using tortoise311 forked from tortoise","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"* Updates\n  * Session work list can be bounded to a set size","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"* Updates\n  * Moved to Tortoise 0.10.0\n  * Last-will message callbacks","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"* Bug fixes\n  * Fix topic return type for `:payload_decode_error` - This is a breaking\n    change if you're using it.\n  * Reduce logging when everything is working fine","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release to hex.","ref":"changelog.html#v0-1-0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}